@@ -258,8 +258,24 @@ class SQLExecute(object):
             try:
                 # 首先尝试这是一个特殊命令，如果返回CommandNotFound，则认为其是一个标准SQL
                 for m_Result in execute(self.SQLCliHandler,  sql):
+                    # 保存之前的运行结果
                     if "type" not in m_Result.keys():
                         m_Result.update({"type": "result"})
+                    if m_Result["type"] == "result" and sql.startswith("__internal__"):
+                        # 对于internal的语句要记录影响的行列信息，同时控制TERMOUT
+                        if m_Result["rows"] is None:
+                            m_Rows = 0
+                        else:
+                            m_Rows = len(m_Result["rows"])
+                        if self.SQLOptions.get('TERMOUT').upper() == 'OFF':
+                            m_Result["rows"] = []
+                            m_Result["title"] = ""
+                        self.LastJsonSQLResult = {"desc": m_Result["headers"],
+                                                  "rows": m_Rows,
+                                                  "elapsed": time.time() - start,
+                                                  "result": m_Result["rows"],
+                                                  "status": 0,
+                                                  "warnings": ""}
                     yield m_Result
             except CommandNotFound:
                 # 进入到SQL执行阶段, 开始执行SQL语句
@@ -288,7 +304,7 @@ class SQLExecute(object):
                         elif "SQL_PREPARE" in m_SQLHint.keys():
                             self.cur.execute(sql)
                         else:
-                            if self.SQLOptions.get("SQL_EXECUTE") == "DIRECT":
+                            if self.SQLOptions.get("SQL_EXECUTE").upper() == "DIRECT":
                                 self.cur.execute_direct(sql)
                             else:
                                 self.cur.execute(sql)
@@ -300,6 +316,7 @@ class SQLExecute(object):
                                 self.get_result(self.cur, rowcount)
                             rowcount = m_FetchedRows
                             if "SQLCLI_DEBUG" in os.environ:
+                                print("!!DEBUG!!  headers=" + str(headers))
                                 if result is not None:
                                     for m_RowPos in range(0, len(result)):
                                         for m_CellPos in range(0, len(result[m_RowPos])):
@@ -320,7 +337,7 @@ class SQLExecute(object):
                                                       "result": result,
                                                       "status": 0,
                                                       "warnings": m_SQLWarnings}
-                            
+
                             # 如果Hint中存在LogFilter，则结果集中过滤指定的输出信息
                             if "LogFilter" in m_SQLHint.keys() and result is not None:
                                 for m_SQLFilter in m_SQLHint["LogFilter"]:
@@ -348,11 +365,16 @@ class SQLExecute(object):
                                             if "SQLCLI_DEBUG" in os.environ:
                                                 print("Apply Mask: " + m_Output +
                                                       " with " + m_SQLMaskPattern + "=>" + m_SQLMaskTarget)
-                                            m_NewOutput = re.sub(m_SQLMaskPattern, m_SQLMaskTarget, m_Output,
-                                                                 re.IGNORECASE)
-                                            if m_NewOutput != m_Output:
-                                                result[i] = tuple(m_NewOutput.split('|||'))
-                                                m_Output = m_NewOutput
+                                            try:
+                                                m_NewOutput = re.sub(m_SQLMaskPattern, m_SQLMaskTarget, m_Output,
+                                                                     re.IGNORECASE)
+                                                if m_NewOutput != m_Output:
+                                                    result[i] = tuple(m_NewOutput.split('|||'))
+                                                    m_Output = m_NewOutput
+                                            except re.error:
+                                                if "SQLCLI_DEBUG" in os.environ:
+                                                    print('traceback.print_exc():\n%s' % traceback.print_exc())
+                                                    print('traceback.format_exc():\n%s' % traceback.format_exc())
                                         else:
                                             if "SQLCLI_DEBUG" in os.environ:
                                                 print("LogMask Hint Error: " + m_SQLHint["LogMask"])
@@ -382,7 +404,7 @@ class SQLExecute(object):
                                 self.SQLOptions.set("SILENT", "ON")
                                 self.SQLOptions.set("TIMING", "OFF")
                                 self.SQLOptions.set("TIME", "OFF")
-                                for m_nLoopPos in range(0, m_LoopTimes):
+                                for m_nLoopPos in range(1, m_LoopTimes):
                                     # 检查Until条件，如果达到Until条件，退出
                                     m_AssertSuccessful = False
                                     for m_Result in \
@@ -612,6 +634,50 @@ class SQLExecute(object):
                             m_row.append(self.SQLOptions.get("DECIMAL_FORMAT") % column)
                         else:
                             m_row.append(column)
+                    elif type(column) == list:
+                        m_ColumnValue = "ARRAY["
+                        for m_nPos in range(0, len(column)):
+                            m_ColumnType = str(type(column[m_nPos]))
+                            if m_nPos == 0:
+                                if m_ColumnType.upper().find('STR') != -1:
+                                    m_ColumnValue = m_ColumnValue + "'" + str(column[m_nPos]) + "'"
+                                elif m_ColumnType.upper().find('SQLDATE') != -1:
+                                    m_ColumnValue = m_ColumnValue + "DATE'" + str(column[m_nPos]) + "'"
+                                elif m_ColumnType.upper().find("FLOAT") != -1:
+                                    m_ColumnValue = m_ColumnValue + \
+                                                    self.SQLOptions.get("FLOAT_FORMAT") % column[m_nPos]
+                                elif m_ColumnType.upper().find("DOUBLE") != -1:
+                                    m_ColumnValue = m_ColumnValue + \
+                                                    self.SQLOptions.get("DOUBLE_FORMAT") % column[m_nPos]
+                                elif m_ColumnType.upper().find("DECIMAL") != -1:
+                                    if self.SQLOptions.get("DECIMAL_FORMAT") != "":
+                                        m_ColumnValue = m_ColumnValue + \
+                                                        self.SQLOptions.get("DECIMAL_FORMAT") % column[m_nPos]
+                                    else:
+                                        m_ColumnValue = m_ColumnValue + column[m_nPos]
+                                else:
+                                    m_ColumnValue = m_ColumnValue + str(column[m_nPos])
+                            else:
+                                if m_ColumnType.upper().find('STR') != -1:
+                                    m_ColumnValue = m_ColumnValue + ",'" + str(column[m_nPos]) + "'"
+                                elif m_ColumnType.upper().find('SQLDATE') != -1:
+                                    m_ColumnValue = m_ColumnValue + ",DATE'" + str(column[m_nPos]) + "'"
+                                elif m_ColumnType.upper().find("FLOAT") != -1:
+                                    m_ColumnValue = m_ColumnValue + "," + \
+                                                    self.SQLOptions.get("FLOAT_FORMAT") % column[m_nPos]
+                                elif m_ColumnType.upper().find("DOUBLE") != -1:
+                                    m_ColumnValue = m_ColumnValue + "," + \
+                                                    self.SQLOptions.get("DOUBLE_FORMAT") % column[m_nPos]
+                                elif m_ColumnType.upper().find("DECIMAL") != -1:
+                                    if self.SQLOptions.get("DECIMAL_FORMAT") != "":
+                                        m_ColumnValue = m_ColumnValue + "," + \
+                                                        self.SQLOptions.get("DECIMAL_FORMAT") % column[m_nPos]
+                                    else:
+                                        m_ColumnValue = m_ColumnValue + "," + column[m_nPos]
+                                else:
+                                    m_ColumnValue = m_ColumnValue + "," + str(column[m_nPos])
+                        m_ColumnValue = m_ColumnValue + "]"
+                        m_row.append(m_ColumnValue)
                     elif type(column) == bytes:
                         # 对于二进制数据，其末尾用0x00表示，这里进行截断
                         column = column.decode()
@@ -648,7 +714,7 @@ class SQLExecute(object):
             status = None
 
         if self.SQLOptions.get('FEEDBACK').upper() == 'ON' and status is not None:
-            status = status.format(rowcount, "" if rowcount == 1 else "s")
+            status = status.format(rowcount, "" if rowcount in [0, 1] else "s")
         else:
             status = None
         return title, result, headers, columntypes, status, m_FetchStatus, rowcount, cursor.warnings
